// $Id$

/***************************************************************************
*   Copyright (C) 2004 by Thomas Fischer                                  *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef BIBTEXENTRY_H
#define BIBTEXENTRY_H

#include <qstringlist.h>

#include <bibtexelement.h>

//class QStringList;

/**
@author Thomas Fischer
*/
class BibTeXEntry : public BibTeXElement
{
    Q_OBJECT
public:
    enum FieldType {ftAbstract, ftAddress, ftAnnote, ftAuthor, ftBookTitle, ftChapter, ftCrossRef, ftEdition, ftEditor, ftHowPublished, ftInstitution, ftISBN, ftISSN, ftJournal, ftKey, ftMonth, ftNote, ftNumber, ftOrganization, ftPages, ftPublisher, ftSchool, ftSeries, ftTitle, ftType, ftURL, ftVolume, ftYear, ftUnknown};
    enum FieldRequireStatus {frsRequired, frsOptional, frsIgnored};

    struct Field
    {
        FieldType fieldType;
        QString key;
        QString value;
    };

    BibTeXEntry( const ElementType elementType, const QString &id, QObject *parent = 0, const char *name = 0 );
    ~BibTeXEntry();

    static QString fieldTypeToString( const FieldType fieldType );
    static QString fieldTypeDescriptionI18N( const FieldType fieldType );
    static FieldType fieldTypeFromString( const QString &fieldTypeString );

    void setElementType( const ElementType elementType );
    ElementType elementType();
    void setId( const QString& id );
    QString id();

    QString toBibTeXString();
    bool containsPattern( QString& pattern, bool caseSensitive = false );

    Field * field( const FieldType fieldType );
    Field * field( const QString& key );
    bool setField( const QString& key, const QString &value );
    bool setField( const FieldType fieldType, const QString &value );
    bool deleteField( const FieldType fieldType );
    bool deleteField( const QString &key );
    static BibTeXEntry::FieldRequireStatus getRequireStatus( BibTeXElement::ElementType elementType, BibTeXEntry::FieldType fieldType );
    int getFieldCount();

    QValueList<Field*>::iterator begin();
    QValueList<Field*>::iterator end();

private:
    ElementType m_elementType;
    QString m_id;
    QValueList<Field*> m_fields;
};

#endif
