// $Id$

/***************************************************************************
*   Copyright (C) 2004 by Thomas Fischer                                  *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qlabel.h>
#include <qtabwidget.h>
#include <qspinbox.h>
#include <qregexp.h>
#include <qcheckbox.h>
#include <qstringlist.h>
#include <qvbox.h>
#include <qlayout.h>
#include <qsplitter.h>
#include <qfont.h>
#include <qobjectlist.h>
#include <kdialogbase.h>
#include <klocale.h>
#include <kcombobox.h>
#include <keditlistbox.h>
#include <kpushbutton.h>
#include <klistview.h>
#include <klineedit.h>
#include <ktextedit.h>
#include <kmessagebox.h>
#include <kiconloader.h>

#include <bibtexentry.h>
#include <kbibtexentrywidgetui.h>
#include <kbibtexfieldlistviewitem.h>
#include <kbibtexselectstring.h>
#include "kbibtexentrywidget.h"

static const QString MonthsI18N[] =
    {
        i18n( "January" ), i18n( "February" ), i18n( "March" ), i18n( "April" ), i18n( "May" ), i18n( "June" ), i18n( "July" ), i18n( "August" ), i18n( "September" ), i18n( "October" ), i18n( "November" ), i18n( "December" )
    };

static const QString Months[] =
    {
        QString( "January" ), QString( "February" ), QString( "March" ), QString( "April" ), QString( "May" ), QString( "June" ), QString( "July" ), QString( "August" ), QString( "September" ), QString( "October" ), QString( "November" ), QString( "December" )
    };

static const QString MonthsTriple[] =
    {
        QString( "jan" ), QString( "feb" ), QString( "mar" ), QString( "apr" ), QString( "may" ), QString( "jun" ), QString( "jul" ), QString( "aug" ), QString( "sep" ), QString( "oct" ), QString( "nov" ), QString( "dec" )
    };

KBibTeXEntryWidget::KBibTeXEntryWidget( BibTeXEntry *entry, BibTeXFile *bibtexfile, QWidget *parent, const char *name )
        : KBibTeXEntryWidgetUI( parent, name ), m_entry( entry ), m_bibtexfile( bibtexfile ), m_enableAllFields( false )
{
    deletedUserFields = new QStringList();

    setupGUI();
    getEntryData();
    getUserFields();
}


KBibTeXEntryWidget::~KBibTeXEntryWidget()
{
    delete deletedUserFields;
}

void KBibTeXEntryWidget::setupGUI()
{
    textLabelEntryType->setText( i18n( "E&ntry Type:" ) );
    textLabelID->setText( i18n( "&Identifier:" ) );
    tabWidgetMain->setTabLabel( tabWidgetMain->page( 0 ), i18n( "Title" ) );
    tabWidgetMain->setTabLabel( tabWidgetMain->page( 1 ), i18n( "Author" ) );
    tabWidgetMain->setTabLabel( tabWidgetMain->page( 2 ), i18n( "Publication" ) );
    tabWidgetMain->setTabLabel( tabWidgetMain->page( 3 ), i18n( "Misc" ) );
    tabWidgetMain->setTabLabel( tabWidgetMain->page( 4 ), i18n( "User Fields" ) );
    textLabelTitle->setText( i18n( "&Title:" ) );
    textLabelBookTitle->setText( i18n( "&Book Title:" ) );
    textLabelSeries->setText( i18n( "&Series:" ) );
    textLabelAuthor->setText( i18n( "&Author:" ) );
    kPushButtonAddAuthor->setText( i18n( "Add Au&thor" ) );
    kPushButtonRemoveAuthor->setText( i18n( "&Remove" ) );
    kPushButtonUpAuthor->setText( i18n( "&Up" ) );
    kPushButtonUpAuthor->setPixmap( KGlobal::iconLoader() ->loadIcon( "up", KIcon::Small ) );
    kPushButtonDownAuthor->setText( i18n( "Do&wn" ) );
    kPushButtonDownAuthor->setPixmap( KGlobal::iconLoader() ->loadIcon( "down", KIcon::Small ) );
    textLabelEditor->setText( i18n( "&Editor:" ) );
    kPushButtonAddEditor->setText( i18n( "Add Edito&r" ) );
    kPushButtonRemoveEditor->setText( i18n( "Re&move" ) );
    kPushButtonUpEditor->setText( i18n( "U&p" ) );
    kPushButtonUpEditor->setPixmap( KGlobal::iconLoader() ->loadIcon( "up", KIcon::Small ) );
    kPushButtonDownEditor->setText( i18n( "Dow&n" ) );
    kPushButtonDownEditor->setPixmap( KGlobal::iconLoader() ->loadIcon( "down", KIcon::Small ) );
    textLabelHowPublished->setText( i18n( "Ho&w Published:" ) );
    textLabelPublisher->setText( i18n( "&Publisher:" ) );
    textLabelOrganization->setText( i18n( "Org&anization:" ) );
    textLabelInstitution->setText( i18n( "Instit&ution:" ) );
    textLabelSchool->setText( i18n( "Schoo&l:" ) );
    textLabelAddress->setText( i18n( "Add&ress:" ) );
    textLabelJournal->setText( i18n( "&Journal:" ) );
    pushButtonJournal->setPixmap( KGlobal::iconLoader() ->loadIcon( "down", KIcon::Small ) );
    textLabelCrossRef->setText( i18n( "Cro&ss Reference:" ) );
    textLabelPages->setText( i18n( "Pa&ges:" ) );
    textLabelEdition->setText( i18n( "E&dition:" ) );
    textLabelNumber->setText( i18n( "Num&ber:" ) );
    textLabelVolume->setText( i18n( "&Volume:" ) );
    textLabelChapter->setText( i18n( "Chap&ter:" ) );
    textLabelMonth->setText( i18n( "Mont&h:" ) );
    textLabelYear->setText( i18n( "&Year:" ) );
    textLabelISBN->setText( i18n( "ISBN:" ) );
    textLabelISSN->setText( i18n( "ISSN:" ) );
    textLabelType->setText( i18n( "&Type:" ) );
    textLabelKey->setText( i18n( "&Key:" ) );
    textLabelNote->setText( i18n( "Not&e:" ) );
    textLabelAnnote->setText( i18n( "&Annote:" ) );
    textLabelURL->setText( i18n( "&URL:" ) );
    kPushButtonURLtoNote->setText( i18n( "&Apply to field 'Note'" ) );
    textLabelAbstract->setText( i18n( "A&bstract:" ) );
    textLabelUserKey->setText( i18n( "&Key:" ) );
    textLabelUserValue->setText( i18n( "&Value:" ) );
    kPushButtonUserAdd->setText( i18n( "&Add" ) );
    kPushButtonUserAdd->setEnabled( FALSE );
    kPushButtonUserDelete->setText( i18n( "&Remove" ) );
    kPushButtonUserDelete->setEnabled( FALSE );
    kListViewUserFields->addColumn( i18n( "Key" ) );
    kListViewUserFields->addColumn( i18n( "Value" ) );
    kListViewUserFields->setAllColumnsShowFocus( true );

    spinBoxEdition->setMinValue( 0 );
    spinBoxEdition->setMaxValue( 10000 );
    spinBoxEdition->setSpecialValueText( i18n( "not set" ) );
    spinBoxVolume->setMinValue( 0 );
    spinBoxVolume->setMaxValue( 10000 );
    spinBoxVolume->setSpecialValueText( i18n( "not set" ) );
    spinBoxYear->setMinValue( 999 );
    spinBoxYear->setMaxValue( 3000 );
    spinBoxYear->setSpecialValueText( i18n( "not set" ) );

    kComboBoxMonth->insertItem( i18n( "not set" ) );
    for ( int i = 0; i < 12; i++ )
        kComboBoxMonth->insertItem( MonthsI18N[ i ] );

    setupEntryTypes();

    connect( kComboBoxEntryType, SIGNAL( activated( int ) ), this, SLOT( slotEntryTypeChanged( int ) ) );
    connect( kListViewUserFields, SIGNAL( executed( QListViewItem* ) ), this, SLOT( slotUserFieldExecute( QListViewItem* ) ) );
    connect( kHistoryComboUserName, SIGNAL( textChanged( const QString& ) ), this, SLOT( slotUserNameChanged( const QString& ) ) );
    connect( kPushButtonUserAdd, SIGNAL( clicked( ) ), this, SLOT( slotUserAddClicked( ) ) );
    connect( kPushButtonUserDelete, SIGNAL( clicked( ) ), this, SLOT( slotUserDeleteClicked( ) ) );
    connect( kPushButtonURLtoNote, SIGNAL( clicked() ), this, SLOT( slotApplyURLtoNode() ) );
    connect( kPushButtonAddAuthor, SIGNAL( clicked() ), this, SLOT( slotAddAuthor() ) );
    connect( kPushButtonRemoveAuthor, SIGNAL( clicked() ), this, SLOT( slotRemoveAuthor() ) );
    connect( kPushButtonUpAuthor, SIGNAL( clicked() ), this, SLOT( slotUpAuthor() ) );
    connect( kPushButtonDownAuthor, SIGNAL( clicked() ), this, SLOT( slotDownAuthor() ) );
    connect( kPushButtonAddEditor, SIGNAL( clicked() ), this, SLOT( slotAddEditor() ) );
    connect( kPushButtonRemoveEditor, SIGNAL( clicked() ), this, SLOT( slotRemoveEditor() ) );
    connect( kPushButtonUpEditor, SIGNAL( clicked() ), this, SLOT( slotUpEditor() ) );
    connect( kPushButtonDownEditor, SIGNAL( clicked() ), this, SLOT( slotDownEditor() ) );
    connect( kLineEditAuthor, SIGNAL( textChanged( const QString & ) ), this, SLOT( slotUpdateAuthorEditorUI() ) );
    connect( kLineEditEditor, SIGNAL( textChanged( const QString & ) ), this, SLOT( slotUpdateAuthorEditorUI() ) );
    connect( kListBoxAuthor, SIGNAL( selectionChanged( ) ), this, SLOT( slotUpdateAuthorEditorUI() ) );
    connect( kListBoxEditor, SIGNAL( selectionChanged( ) ), this, SLOT( slotUpdateAuthorEditorUI() ) );
    connect( kLineEditAuthor, SIGNAL( returnPressed( ) ), this, SLOT( slotAddAuthor() ) );
    connect( kLineEditEditor, SIGNAL( returnPressed( ) ), this, SLOT( slotAddEditor() ) );
    connect( pushButtonJournal, SIGNAL( clicked( ) ), this, SLOT( slotSelectStringJournal() ) );

    slotUpdateAuthorEditorUI();
}

void KBibTeXEntryWidget::setupEntryTypes()
{
    int i;
    BibTeXElement::ElementType elementType;

    i = ( int ) BibTeXEntry::etArticle;
    elementType = ( BibTeXEntry::ElementType ) i;
    while ( elementType != BibTeXEntry::etUnknown )
    {
        kComboBoxEntryType->insertItem( BibTeXElement::elementTypeToString( elementType ) );
        elementType = ( BibTeXElement::ElementType ) ++i;
    }
}

void KBibTeXEntryWidget::getEntryData()
{
    bool ok;
    BibTeXEntry::Field *field;

    kComboBoxEntryType->setCurrentItem( m_entry->elementType() - ( int ) BibTeXEntry::etArticle );
    lineEditID->setText( m_entry->id() );
    field = m_entry->field( BibTeXEntry::ftTitle );
    if ( field )
        lineEditTitle->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftBookTitle );
    if ( field )
        lineEditBookTitle->setText( field ->value );
    field = m_entry->field( BibTeXEntry::ftSeries );
    if ( field )
        lineEditSeries->setText( field ->value );
    field = m_entry->field( BibTeXEntry::ftHowPublished );
    if ( field )
        kLineEditHowPublished->setText( field ->value );
    field = m_entry->field( BibTeXEntry::ftPublisher );
    if ( field )
        kLineEditPublisher->setText( field ->value );
    field = m_entry->field( BibTeXEntry::ftOrganization );
    if ( field )
        kLineEditOrganization->setText( field ->value );
    field = m_entry->field( BibTeXEntry::ftInstitution );
    if ( field )
        kLineEditInstitution->setText( field ->value );
    field = m_entry->field( BibTeXEntry::ftSchool );
    if ( field )
        kLineEditSchool->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftAddress );
    if ( field )
        kLineEditAddress->setText( field ->value );
    field = m_entry->field( BibTeXEntry::ftJournal );
    if ( field )
        kLineEditJournal->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftCrossRef );
    if ( field )
        kLineEditCrossRef->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftISBN );
    if ( field )
        kLineEditISBN->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftISSN );
    if ( field )
        kLineEditISSN->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftPages );
    if ( field )
        lineEditPages->setText( field ->value );
    field = m_entry->field( BibTeXEntry::ftNumber );
    if ( field )
        kLineEditNumber->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftChapter );
    if ( field )
        lineEditChapter->setText( field ->value );
    field = m_entry->field( BibTeXEntry::ftType );
    if ( field )
        lineEditType->setText( field ->value );
    field = m_entry->field( BibTeXEntry::ftKey );
    if ( field )
        lineEditKey->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftNote );
    if ( field )
        lineEditNote->setText( field ->value );
    field = m_entry->field( BibTeXEntry::ftAnnote );
    if ( field )
        lineEditAnnote->setText( field ->value );
    field = m_entry->field( BibTeXEntry::ftAbstract );
    if ( field )
        kTextEditAbstract->setText( field ->value );
    field = m_entry->field( BibTeXEntry::ftURL );
    if ( field )
        lineEditURL->setText( field ->value );

    field = m_entry->field( BibTeXEntry::ftEdition );
    int edition = 0;
    if ( field && ( ( edition = field->value.toInt( &ok ) ) ) && ok && ( edition >= 1 ) && ( edition <= spinBoxEdition->maxValue() ) )
        spinBoxEdition->setValue( edition );
    else
        spinBoxEdition->setValue( spinBoxEdition->minValue() );

    field = m_entry->field( BibTeXEntry::ftVolume );
    int volume = 0;
    if ( field && ( ( volume = field->value.toInt( &ok ) ) ) && ok && ( volume >= 1 ) && ( volume <= spinBoxVolume->maxValue() ) )
        spinBoxVolume->setValue( volume );
    else
        spinBoxVolume->setValue( spinBoxVolume->minValue() );

    field = m_entry->field( BibTeXEntry::ftYear );
    int year = 0;
    if ( field && ( ( year = field->value.toInt( &ok ) ) ) && ok && ( year >= 1000 ) && ( year <= spinBoxYear->maxValue() ) )
        spinBoxYear->setValue( year );
    else
        spinBoxYear->setValue( spinBoxYear->minValue() );

    field = m_entry->field( BibTeXEntry::ftAuthor );
    if ( field )
        kListBoxAuthor->insertStringList( QStringList::split( QRegExp( "\\s+(,|and|&)\\s+", false ), field ->value ) );
    field = m_entry->field( BibTeXEntry::ftEditor );
    if ( field )
        kListBoxEditor->insertStringList( QStringList::split( QRegExp( "\\s+(,|and|&)\\s+", false ), field ->value ) );

    field = m_entry->field( BibTeXEntry::ftMonth );
    if ( field )
        for ( int i = 0; i < 12; i++ )
        {
            QString entryMonth = field ->value.lower();
            if ( ( entryMonth.compare( Months[ i ].lower() ) == 0 ) || ( entryMonth.compare( MonthsTriple[ i ].lower() ) == 0 ) || ( entryMonth.compare( MonthsI18N[ i ].lower() ) == 0 ) )
                kComboBoxMonth->setCurrentItem( i + 1 );
        }

    changeElementType( m_entry->elementType() );
}

bool KBibTeXEntryWidget::setEntryDataSpinBox( QSpinBox *sbx, BibTeXEntry::FieldType fieldType )
{
    if ( sbx->value() > sbx->minValue() )
    {
        QString text = QString::number( sbx->value() );
        m_entry->setField( fieldType, text );
        return true;
    }
    else
    {
        m_entry->deleteField( fieldType );
        return false;
    }
}

bool KBibTeXEntryWidget::setEntryDataText( QString text, BibTeXEntry::FieldType fieldType )
{
    if ( !text.isEmpty() )
        m_entry->setField( fieldType, text );
    else
        m_entry->deleteField( fieldType );

    return ( BibTeXEntry::getRequireStatus( ( BibTeXElement::ElementType ) ( ( int ) BibTeXEntry::etArticle + kComboBoxEntryType->currentItem() ), fieldType ) != BibTeXEntry::frsRequired ) || !text.isEmpty();
}

bool KBibTeXEntryWidget::setEntryData()
{
    bool result = true;
    QString text;

    m_entry->setElementType( ( BibTeXElement::ElementType ) ( ( int ) BibTeXEntry::etArticle + kComboBoxEntryType->currentItem() ) );
    text = lineEditID->text();
    m_entry->setId( text );

    setEntryDataText( lineEditTitle->text(), BibTeXEntry::ftTitle );
    setEntryDataText( lineEditBookTitle->text(), BibTeXEntry::ftBookTitle );
    setEntryDataText( lineEditSeries->text(), BibTeXEntry::ftSeries );

    setEntryDataText( kLineEditHowPublished->text(), BibTeXEntry::ftHowPublished );
    setEntryDataText( kLineEditPublisher->text(), BibTeXEntry::ftPublisher );
    setEntryDataText( kLineEditOrganization->text(), BibTeXEntry::ftOrganization );
    setEntryDataText( kLineEditInstitution->text(), BibTeXEntry::ftInstitution );
    setEntryDataText( kLineEditSchool->text(), BibTeXEntry::ftSchool );
    setEntryDataText( kLineEditAddress->text(), BibTeXEntry::ftAddress );
    setEntryDataText( kLineEditJournal->text(), BibTeXEntry::ftJournal );
    setEntryDataText( kLineEditCrossRef->text(), BibTeXEntry::ftCrossRef );
    setEntryDataText( kLineEditISBN->text(), BibTeXEntry::ftISBN );
    setEntryDataText( kLineEditISSN->text(), BibTeXEntry::ftISSN );
    setEntryDataText( lineEditPages->text(), BibTeXEntry::ftPages );
    setEntryDataText( lineEditChapter->text(), BibTeXEntry::ftChapter );
    setEntryDataText( kLineEditNumber->text(), BibTeXEntry::ftNumber );
    setEntryDataText( lineEditType->text(), BibTeXEntry::ftType );
    setEntryDataText( lineEditKey->text(), BibTeXEntry::ftKey );
    setEntryDataText( lineEditNote->text(), BibTeXEntry::ftNote );
    setEntryDataText( lineEditAnnote->text(), BibTeXEntry::ftAnnote );
    setEntryDataText( kTextEditAbstract->text(), BibTeXEntry::ftAbstract );
    setEntryDataText( lineEditURL->text(), BibTeXEntry::ftURL );

    setEntryDataSpinBox( spinBoxEdition, BibTeXEntry::ftEdition );
    setEntryDataSpinBox( spinBoxVolume, BibTeXEntry::ftVolume );
    setEntryDataSpinBox( spinBoxYear, BibTeXEntry::ftYear );

    QListBoxItem *item = kListBoxAuthor->firstItem();
    QStringList items;
    while ( item )
    {
        items << item->text();
        item = item->next();
    }
    setEntryDataText( items.join( " and " ), BibTeXEntry::ftAuthor );
    items.clear();
    item = kListBoxEditor->firstItem();
    while ( item )
    {
        items << item->text( );
        item = item->next();
    }
    setEntryDataText( items.join( " and " ), BibTeXEntry::ftEditor );

    if ( kComboBoxMonth->currentItem() > 0 )
    {
        QString text = QString( MonthsTriple[ kComboBoxMonth->currentItem() - 1 ] );
        m_entry->setField( BibTeXEntry::ftMonth, text );
    }
    else
        m_entry->deleteField( BibTeXEntry::ftMonth );

    for ( QStringList::iterator it = deletedUserFields->begin(); it != deletedUserFields->end(); it++ )
        m_entry->deleteField( *it );
    for ( QListViewItemIterator it( kListViewUserFields ); it.current(); it++ )
    {
        m_entry->deleteField( it.current() ->text( 0 ) );
        m_entry->setField( it.current() ->text( 0 ), it.current() ->text( 1 ) );
    }

    return result;
}

QDialog::DialogCode KBibTeXEntryWidget::execute( BibTeXEntry *entry, BibTeXFile *bibtexfile, const QString& caption, QWidget *parent, const char *name )
{
    KDialogBase * dlg = new KDialogBase( parent, name, true, caption, KDialogBase::Ok | KDialogBase::Cancel );
    QVBox *vbox = new QVBox( dlg );
    KBibTeXEntryWidget* ui = new KBibTeXEntryWidget( entry, bibtexfile, vbox, "kbibtexentrywidget" );
    connect( dlg, SIGNAL( okClicked() ), ui, SLOT( slotOk() ) );

    QCheckBox *cb = new QCheckBox( i18n( "Enable all &fields for editing" ), vbox );
    connect( cb, SIGNAL( toggled( bool ) ), ui, SLOT( slotEnableAllFields( bool ) ) );

    dlg->setMainWidget( vbox );
    QDialog::DialogCode result = ( QDialog::DialogCode ) dlg->exec();

    delete( ui );
    delete( dlg );

    return result;
}

void KBibTeXEntryWidget::slotOk()
{
    setEntryData();
}

void KBibTeXEntryWidget::changeElementType( BibTeXElement::ElementType elementType )
{
    setWidgetStatus( lineEditTitle, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftTitle ) );
    setWidgetStatus( lineEditBookTitle, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftBookTitle ) );
    setWidgetStatus( lineEditSeries, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftSeries ) );
    setWidgetStatus( frameAuthor, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftAuthor ) );
    setWidgetStatus( frameEditor, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftEditor ) );
    setWidgetStatus( kLineEditHowPublished, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftHowPublished ) );
    setWidgetStatus( kLineEditOrganization, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftOrganization ) );
    setWidgetStatus( kLineEditSchool, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftSchool ) );
    setWidgetStatus( kLineEditJournal, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftJournal ) );
    setWidgetStatus( pushButtonJournal, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftJournal ) );
    setWidgetStatus( kLineEditCrossRef, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftCrossRef ) );
    setWidgetStatus( kLineEditPublisher, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftPublisher ) );
    setWidgetStatus( kLineEditInstitution, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftInstitution ) );
    setWidgetStatus( kLineEditAddress, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftAddress ) );
    setWidgetStatus( kLineEditISBN, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftISBN ) );
    setWidgetStatus( kLineEditISSN, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftISSN ) );
    setWidgetStatus( lineEditPages, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftPages ) );
    setWidgetStatus( spinBoxEdition, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftEdition ) );
    setWidgetStatus( kLineEditNumber, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftNumber ) );
    setWidgetStatus( spinBoxVolume, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftVolume ) );
    setWidgetStatus( lineEditChapter, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftChapter ) );
    setWidgetStatus( kComboBoxMonth, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftMonth ) );
    setWidgetStatus( spinBoxYear, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftYear ) );
    setWidgetStatus( lineEditType, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftType ) );
    setWidgetStatus( lineEditKey, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftKey ) );
    setWidgetStatus( lineEditNote, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftNote ) );
    setWidgetStatus( lineEditAnnote, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftAnnote ) );
    //setWidgetStatus( lineEditURL, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftURL ) );
}

void KBibTeXEntryWidget::setWidgetStatus( QWidget* widget, BibTeXEntry::FieldRequireStatus requiredStatus )
{
    widget->setEnabled( ( requiredStatus != BibTeXEntry::frsIgnored ) || m_enableAllFields );

    QWidget *parent = widget->parentWidget();
    for ( QObjectListIterator it( *( parent->children() ) ); it.current(); ++it )
    {
        QLabel *neighbour = dynamic_cast<QLabel*>( it.current() );
        if ( neighbour && ( neighbour->buddy() == widget ) )
        {
            QFont font = parent->font();
            switch ( requiredStatus )
            {
            case BibTeXEntry::frsRequired:
                font.setBold( true );
                break;
            default:
                break;
            }
            neighbour->setFont( font );
        }
    }
}

void KBibTeXEntryWidget::slotEntryTypeChanged( int index )
{
    changeElementType( ( BibTeXElement::ElementType ) ( ( int ) BibTeXEntry::etArticle + index ) );
}

void KBibTeXEntryWidget::slotEnableAllFields( bool enable )
{
    m_enableAllFields = enable;
    changeElementType( ( BibTeXElement::ElementType ) ( ( int ) BibTeXEntry::etArticle + kComboBoxEntryType->currentItem() ) );
}

void KBibTeXEntryWidget::getUserFields()
{
    for ( QValueList<BibTeXEntry::Field*>::iterator it = m_entry->begin(); it != m_entry->end(); it++ )
    {
        if ( ( *it ) ->fieldType == BibTeXEntry::ftUnknown )
            new KBibTeXFieldListViewItem( ( *it ) ->fieldType, ( *it ) ->key, ( *it ) ->value, kListViewUserFields );
    }
}

void KBibTeXEntryWidget::slotUserFieldExecute( QListViewItem* item )
{
    KBibTeXFieldListViewItem * flvi = dynamic_cast<KBibTeXFieldListViewItem*>( item );
    kHistoryComboUserName->setCurrentText( flvi->fieldName() );
    kTextEditUserValue->setText( flvi->fieldValue() );
}

void KBibTeXEntryWidget::slotUserNameChanged( const QString& newText )
{
    QListViewItem * item = kListViewUserFields->findItem( newText, 0 ) ;
    bool contains = item != NULL;
    bool isUnknown = BibTeXEntry::fieldTypeFromString( newText ) == BibTeXEntry::ftUnknown;

    if ( contains )
        kListViewUserFields->setSelected( item, TRUE );
    kPushButtonUserDelete->setEnabled( contains );
    kPushButtonUserAdd->setEnabled( !newText.isEmpty() && isUnknown );
    kPushButtonUserAdd->setText( contains ? i18n( "&Apply" ) : i18n( "&Add" ) );
}

void KBibTeXEntryWidget::slotUserAddClicked( )
{
    KBibTeXFieldListViewItem * flvi = dynamic_cast<KBibTeXFieldListViewItem*>( kListViewUserFields->findItem( kHistoryComboUserName->currentText(), 0 ) );

    if ( flvi != NULL )
    {
        flvi->setFieldValue( kTextEditUserValue->text() );
        flvi->updateItem();
    }
    else
        kListViewUserFields->setSelected( new KBibTeXFieldListViewItem( BibTeXEntry::fieldTypeFromString( kHistoryComboUserName->currentText() ), kHistoryComboUserName->currentText(), kTextEditUserValue->text(), kListViewUserFields ), true );

    kPushButtonUserAdd->setText( i18n( "&Apply" ) );
    kPushButtonUserDelete->setEnabled( TRUE );
}

void KBibTeXEntryWidget::slotUserDeleteClicked()
{
    QListViewItem * item = kListViewUserFields->findItem( kHistoryComboUserName->currentText(), 0 );

    if ( item != NULL )
    {
        deletedUserFields->append( item->text( 0 ) );
        kListViewUserFields->removeItem( item );
        kPushButtonUserAdd->setText( i18n( "&Add" ) );
        kPushButtonUserDelete->setEnabled( FALSE );
    }
}

void KBibTeXEntryWidget::slotApplyURLtoNode()
{
    lineEditNote->setText( "\\url{" + lineEditURL->text() + "}" );
}

void KBibTeXEntryWidget::slotAddAuthor()
{
    kListBoxAuthor->insertItem( kLineEditAuthor->text() );
    kLineEditAuthor->setText( "" );
    slotUpdateAuthorEditorUI();
}

void KBibTeXEntryWidget::slotRemoveAuthor()
{
    kListBoxAuthor->removeItem( kListBoxAuthor->currentItem() );
    slotUpdateAuthorEditorUI();
}

void KBibTeXEntryWidget::slotAddEditor()
{
    kListBoxEditor->insertItem( kLineEditEditor->text() );
    kLineEditEditor->setText( "" );
    slotUpdateAuthorEditorUI();
}

void KBibTeXEntryWidget::slotRemoveEditor( )
{
    kListBoxEditor->removeItem( kListBoxEditor->currentItem() );
    slotUpdateAuthorEditorUI();
}

void KBibTeXEntryWidget::slotUpAuthor()
{
    int pos = kListBoxAuthor->currentItem();
    if ( pos > 0 )
    {
        QString text = kListBoxAuthor->currentText();
        kListBoxAuthor->removeItem( pos );
        kListBoxAuthor->insertItem( text, pos - 1 );
        kListBoxAuthor->setSelected( pos - 1, true );
        slotUpdateAuthorEditorUI();
    }
}

void KBibTeXEntryWidget::slotDownAuthor()
{
    int pos = kListBoxAuthor->currentItem();
    if ( ( pos > -1 ) && ( pos < ( int ) kListBoxAuthor->count() - 1 ) )
    {
        QString text = kListBoxAuthor->currentText();
        kListBoxAuthor->removeItem( pos );
        kListBoxAuthor->insertItem( text, pos + 1 );
        kListBoxAuthor->setSelected( pos + 1, true );
        slotUpdateAuthorEditorUI();
    }
}

void KBibTeXEntryWidget::slotUpEditor()
{
    int pos = kListBoxEditor->currentItem();
    if ( pos > 0 )
    {
        QString text = kListBoxEditor->currentText();
        kListBoxEditor->removeItem( pos );
        kListBoxEditor->insertItem( text, pos - 1 );
        kListBoxEditor->setSelected( pos - 1, true );
        slotUpdateAuthorEditorUI();
    }
}

void KBibTeXEntryWidget::slotDownEditor()
{
    int pos = kListBoxEditor->currentItem();
    if ( ( pos > -1 ) && ( pos < ( int ) kListBoxEditor->count() - 1 ) )
    {
        QString text = kListBoxEditor->currentText();
        kListBoxEditor->removeItem( pos );
        kListBoxEditor->insertItem( text, pos + 1 );
        kListBoxEditor->setSelected( pos + 1, true );
        slotUpdateAuthorEditorUI();
    }
}

void KBibTeXEntryWidget::slotUpdateAuthorEditorUI()
{
    kPushButtonAddAuthor->setEnabled( !kLineEditAuthor->text().isEmpty() );
    kPushButtonRemoveAuthor->setEnabled( kListBoxAuthor->currentItem() > -1 );
    kPushButtonUpAuthor->setEnabled( kListBoxAuthor->currentItem() > 0 );
    kPushButtonDownAuthor->setEnabled( ( kListBoxAuthor->currentItem() > -1 ) && ( kListBoxAuthor->currentItem() < ( int ) kListBoxAuthor->count() - 1 ) );
    kPushButtonAddEditor->setEnabled( !kLineEditEditor->text().isEmpty() );
    kPushButtonRemoveEditor->setEnabled( kListBoxEditor->currentItem() > -1 );
    kPushButtonUpEditor->setEnabled( kListBoxEditor->currentItem() > 0 );
    kPushButtonDownEditor->setEnabled( ( kListBoxEditor->currentItem() > -1 ) && ( kListBoxEditor->currentItem() < ( int ) kListBoxEditor->count() - 1 ) );
}

void KBibTeXEntryWidget::slotSelectStringJournal()
{
    QString macro = KBibTeXSelectString::execute( m_bibtexfile, this );
    if ( !macro.isNull() )
        kLineEditJournal->setText( macro );
}

#include "kbibtexentrywidget.moc"
